#include <windows.h>
#include "buffer.h"
#include "multiasmlib.h"

void disasm_demo();
void asm_demo();

void main()
{
	disasm_demo();
	asm_demo();

	ExitProcess(0);
}

void disasm_demo()
{
	BYTE code1[] = {
		0x90,       // NOP
		0xEB, 0x02, // JMP $+2
		0x33, 0xC0, // XOR EAX,EAX
		0x40        // INC EAX
	};
	BYTE code2[] = {
		0x90,       // NOP
		0xFF, 0xFF  // No such command
	};
	char *text;
	DISASM_OPTIONS disasm_options;
	char errorstr[1025];

	disasm_options.disasm_label = 1;    // use labels
	disasm_options.disasm_hex = 3;      // 0xFFFE
	disasm_options.disasm_labelgen = 1; //L_[address]

	text = MUADisassemble(code1, 0x401000, sizeof(code1), NULL, &disasm_options, errorstr);
	if(text)
	{
		MessageBox(NULL, text, "DISASM TEST 1", MB_OK);
		MUADisassembleFree(text);
	}
	else
		MessageBox(NULL, errorstr, "DISASM TEST 1", MB_ICONHAND);

	// REMEMBER!
	// After you run MUADisassemble, the binary code (the first parameter) gets modified.
	// That's the way the function works. If you need it unmodified, pass a copy!

	text = MUADisassemble(code2, 0x401000, sizeof(code2), NULL, &disasm_options, errorstr);
	if(text)
	{
		MessageBox(NULL, text, "DISASM TEST 2", MB_OK);
		MUADisassembleFree(text);
	}
	else
		MessageBox(NULL, errorstr, "DISASM TEST 2", MB_ICONHAND);
}

void asm_demo()
{
	char text1[] = 
		"MOV EAX,$$3000\n" // base address + 3000
		"MOV EAX,@f\n"
		"\"\\x12\\x34\"\n" // "\x12\x34"
		"@@: JMP SHORT @b";
	char text2[] = 
		"MOV bla bla bla"; // Invalid
	BYTE *code;
	DWORD size;
	int error_offset;
	char errorstr[1025];
	char buffer[100];
	UINT i;

	code = MUAAssemble(text1, 0x401000, &size, 0x400000, &error_offset, errorstr);
	if(code)
	{
		for(i=0; i<size; i++)
			wsprintf(&buffer[i*3], "%02X ", code[i]);

		MessageBox(NULL, buffer, "ASM TEST 1", MB_OK);

		MUAAssembleFree(code);
	}
	else
	{
		MessageBox(NULL, errorstr, "ASM TEST 1", MB_ICONHAND);

		wsprintf(buffer, "error_offset = %d", error_offset);

		MessageBox(NULL, buffer, "ASM TEST 1", MB_ICONHAND);
	}

	// REMEMBER!
	// After you run MUAAssemble, the assembly text (the first parameter) gets modified.
	// That's the way the function works. If you need it unmodified, pass a copy!

	code = MUAAssemble(text2, 0x401000, &size, 0x400000, &error_offset, errorstr);
	if(code)
	{
		for(i=0; i<size; i++)
			wsprintf(&buffer[i*3], "%02X ", code[i]);

		MessageBox(NULL, buffer, "ASM TEST 2", MB_OK);

		MUAAssembleFree(code);
	}
	else
	{
		MessageBox(NULL, errorstr, "ASM TEST 2", MB_ICONHAND);

		wsprintf(buffer, "error_offset = %d", error_offset);

		MessageBox(NULL, buffer, "ASM TEST 2", MB_ICONHAND);
	}
}
