#ifndef _MULTIASMLIB_H_
#define _MULTIASMLIB_H_

typedef struct {
	int ideal;                // Force IDEAL decoding mode
	int lowercase;            // Force lowercase display
	int tabarguments;         // Tab between mnemonic and arguments
	int extraspace;           // Extra space between arguments
	int putdefseg;            // Display default segments in listing
	int showmemsize;          // Always show memory size
	int shownear;             // Show NEAR modifiers
	int shortstringcmds;      // Use short form of string commands
	int sizesens;             // How to decode size-sensitive mnemonics
	int symbolic;             // Show symbolic addresses in disasm
	int farcalls;             // Accept far calls, returns & addresses
	int decodevxd;            // Decode VxD calls (Win95/98)
	int privileged;           // Accept privileged commands
	int iocommand;            // Accept I/O commands
	int badshift;             // Accept shift out of range 1..31
	int extraprefix;          // Accept superfluous prefixes
	int lockedbus;            // Accept LOCK prefixes
	int stackalign;           // Accept unaligned stack operations
	int iswindowsnt;          // When checking for dangers, assume NT
} ODBG_DISASM_OPTIONS;

typedef struct {
	int disasm_label;         // nonzero to use labels
	int disasm_hex;           // 0: FFFE, 1: 0FFFE, 2: 0FFFEh ,3: 0xFFFE
	int disasm_labelgen;      // 0: L[counter], 1: L_[address], 2: L_[pLabelPerfix]_[counter]
} DISASM_OPTIONS;

// FUNCTION
// MUASetOdbgDisasmOptions
//
// DESCRIPTION
// Sets OllyDbg disassembler flags. By default, all flags are zero.
//
// PARAMETERS
// ODBG_DISASM_OPTIONS *p_odbg_disasm_options
//   OllyDbg disassembler flags, see ODBG_DISASM_OPTIONS struct for details.

void MUASetOdbgDisasmOptions(ODBG_DISASM_OPTIONS *p_odbg_disasm_options);

// FUNCTION
// MUADisassemble
//
// DESCRIPTION
// Disassembles binary code to text, compatible with Multiline Ultimate Assembler.
// Note: it doesn't include the block address (<00401000>).
//
// PARAMETERS
// BYTE *pCode
//   A pointer to the binary code.
//   Note: must be writable, and will be altered!
// DWORD dwAddress
//   The address (instruction pointer) of the code.
// DWORD dwSize
//   The size of the code.
// char *pLabelPerfix
//   Used with the disasm_labelgen=2 option.
//   Note: must be writable! If it contains invalid characters, they will be replaced.
// DISASM_OPTIONS *p_options:
//   Disassembler options, see DISASM_OPTIONS struct for details.
// char *lpError
//   In case of an error, recieves the error string. Must be at least 1025 characters long.
//
// RETURN
// char *
//   A pointer to the disassembled text in case of success. Free with MUADisassembleFree.
//   NULL in case of failure.

char *MUADisassemble(BYTE *pCode, DWORD dwAddress, DWORD dwSize, char *pLabelPerfix, DISASM_OPTIONS *p_options, char *lpError);

// FUNCTION
// MUADisassembleFree
//
// DESCRIPTION
// Frees the pointer returned by MUADisassemble.
//
// PARAMETERS
// char *lpText
//   The pointer returned by MUADisassemble.
//
// RETURN
// BOOL
//   Nonzero in case of a success.
//   Zero in case of failure.

BOOL MUADisassembleFree(char *lpText);

// FUNCTION
// MUAAssemble
//
// DESCRIPTION
// Assembles text, compatible with Multiline Ultimate Assembler, to binary code.
// Note: do not include the block address (<00401000>).
//
// PARAMETERS
// char *lpText
//   A pointer to the zero terminated assembly text.
//   Note: must be writable, and will be altered!
// DWORD dwAddress
//   The address (instruction pointer) of the code.
// DWORD *pdwSize
//   Recieves the size of the assembled binary code.
// DWORD dwBaseAddress
//   The base address used for relative RVA addresses ($$1000).
//   See Multiline Ultimate Assembler help for more details.
// int *pnErrorOffset
//   In case of an error, recieves the offset of the error in the assembly text. Optional.
// char *lpError
//   In case of an error, recieves the error string. Must be at least 1025 characters long.
//
// RETURN
// BYTE *
//   A pointer to the assembled binary code in case of success. Free with MUAAssembleFree.
//   NULL in case of failure.

BYTE *MUAAssemble(char *lpText, DWORD dwAddress, DWORD *pdwSize, DWORD dwBaseAddress, int *pnErrorOffset, char *lpError);

// FUNCTION
// MUAAssembleFree
//
// DESCRIPTION
// Frees the pointer returned by MUAAssemble.
//
// PARAMETERS
// BYTE *pAsm
//   The pointer returned by MUAAssemble.
//
// RETURN
// BOOL
//   Nonzero in case of a success.
//   Zero in case of failure.

BOOL MUAAssembleFree(BYTE *pAsm);

#endif // _MULTIASMLIB_H_
